@@ -1,10 +1,8 @@
 """Declares :class:`BaseResource`."""
-from types import FunctionType
 from typing import Union
 
 from fastapi import Request
 from fastapi.responses import Response
-from fastapi.responses import JSONResponse
 
 from ..resourceendpointset import ResourceEndpointSet
 
